////////////////////////////////////////////////////////////////////////////////
//
//	Various tools shared by SOLAR softwares
//
////////////////////////////////////////////////////////////////////////////////

#define	MODULE_NAME	"ERRORS"

#include "TOOLS.h"

#define	MESSAGE_LENGTH	4096

static	trace_level = 1;

static	FILE *	log = NULL;		// Log file descriptor


////////////////////////////////////////////////////////////////////////////////
//
//	Test fatal error
//
int	Error (t_err err)
{
	return err >= ERROR;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Test warning
//
int	Warning (t_err err)
{	
	return (err >= WARNING ) && (err < ERROR);
}


////////////////////////////////////////////////////////////////////////////////
//
//	Update trace level
//
void 	Set_trace_level (int level) 
{
	trace_level = level;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Get trace level
//
int	Get_trace_level()
{
	return trace_level;
}

////////////////////////////////////////////////////////////////////////////////
//
//	Redirect stdout to logfile
//
t_err	Set_logfile (char * filename)
{
	char *		fname = FNAME ("Set_logfile");
	t_err		error = OK;
	FILE *		output = NULL;

	if ((output = fopen (filename, "w")) == NULL) {

		Error_log (fname, "Error creating logfile %s", filename);
		error = ERR_open;
		goto EXIT;
	}

	printf ("Logfile : %s\n", filename);

	fflush (stdout);

	log = output;

EXIT:	return error;
}


////////////////////////////////////////////////////////////////////////////////
//
//	Log messages d'erreur
//
void	Error_log (char * fname, char * format, ...)
{
	char		buffer [MESSAGE_LENGTH];
	va_list		ptr;

	va_start (ptr, format);
	vsnprintf (buffer, MESSAGE_LENGTH -1, format, ptr);
	va_end (ptr);
	
	fprintf (log, "E | %-30.30s | %s\n", fname, buffer);
	fflush (log);
}


////////////////////////////////////////////////////////////////////////////////
//
//	Log warnings
//
void	Warning_log (char * fname, char * format, ...)
{
	char		buffer [MESSAGE_LENGTH];
	va_list		ptr;

	va_start (ptr, format);
	vsnprintf (buffer, MESSAGE_LENGTH -1, format, ptr);
	va_end (ptr);
	
	fprintf (log, "W | %-30.30s | %s\n", fname, buffer);
	fflush (log);
}


////////////////////////////////////////////////////////////////////////////////
//
//	Log messages d'information
//
void	Message_log (int level, char * fname, char * format, ...)
{
	char		buffer [MESSAGE_LENGTH];
	va_list		ptr;

	if (trace_level >= level) {

		va_start (ptr, format);
		vsnprintf (buffer, MESSAGE_LENGTH -1, format, ptr);
		va_end (ptr);
		
		fprintf (log, "%d | %-30.30s | %s\n", level, fname, buffer);
		fflush (log);
	}
}


////////////////////////////////////////////////////////////////////////////////
//
//	Set default logfile to stdout
//
//	NOTE : this function is called when library is loaded
//
__attribute__((constructor)) void Reset_logfile()
{
	if (log == NULL) log = stdout;
}

__attribute__((destructor)) void Close_logfile()
{
	char * fname = FNAME ("Close_logfile");

	if (log != stdout) {
		Message_log (1, fname, "Close logfile");
		fclose (log);
	}
}
